@@ -58,6 +58,7 @@ bool AppleRAIDMirrorSet::init()
     arRebuildThreadCall = 0;
     arSetCompleteThreadCall = 0;
     arExpectingLiveAdd = 0;
+    arMaxReadRequestFactor = 32;	// with the default 32KB blocksize -> 1 MB
 
     queue_init(&arFailedRequestQueue);
 
@@ -85,6 +86,9 @@ void AppleRAIDMirrorSet::free(void)
     if (arSetCompleteThreadCall) thread_call_free(arSetCompleteThreadCall);
     arSetCompleteThreadCall = 0;
 
+    if (arLastSeek) IODelete(arLastSeek, UInt64, arLastAllocCount);
+    if (arSkippedIOCount) IODelete(arSkippedIOCount, UInt64, arLastAllocCount);
+
     assert(queue_empty(&arFailedRequestQueue));
 
     super::free();
@@ -132,6 +136,19 @@ bool AppleRAIDMirrorSet::resizeSet(UInt32 newMemberCount)
 {
     UInt32 oldMemberCount = arMemberCount;
 
+    // if downsizing, just hold on to the extra space
+    if (arLastAllocCount < newMemberCount) {
+	if (arLastSeek) IODelete(arLastSeek, UInt64, arLastAllocCount);
+	arLastSeek = IONew(UInt64, newMemberCount);
+	if (!arLastSeek) return false;
+
+	if (arSkippedIOCount) IODelete(arSkippedIOCount, UInt64, arLastAllocCount);
+	arSkippedIOCount = IONew(UInt64, newMemberCount);
+	if (!arSkippedIOCount) return false;
+    }
+    bzero(arLastSeek, sizeof(UInt64) * newMemberCount);
+    bzero(arSkippedIOCount, sizeof(UInt64) * newMemberCount);
+
     if (super::resizeSet(newMemberCount) == false) return false;
 
     if (oldMemberCount && arMemberCount > oldMemberCount) arExpectingLiveAdd += arMemberCount - oldMemberCount;
@@ -198,6 +215,104 @@ bool AppleRAIDMirrorSet::bumpOnError(void)
 //8888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888
 //8888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888
 
+void AppleRAIDMirrorSet::activeReadMembers(AppleRAIDMember ** activeMembers, UInt64 byteStart, UInt32 byteCount)
+{
+    // this code try's to do three things:
+    //   1) send large single sequential i/o requests to each disk		(arMaxReadRequestFactor)
+    //   2) send i/o requests to the disk with the smallest seek distance	(arLastSeek)
+    //   3) balance the number of i/o requests between the available drives	(arSkippedIOCount)
+    //
+    // this code completely ignores the effects of writes on the head position since writes move
+    // the heads on all disks.  if the disk is doing track caching then ignoring the writes can
+    // still get us to a disk that may have that data already cached.
+    //
+    // note that arLastSeek is the last previously scheduled head position, the head may not
+    // be anywhere near there yet, hence this code can schedule multiple future i/o requests
+
+#define isOnline(member) ((UInt32)(member) >= 0x1000)    
+#define isOffline(member) ((UInt32)(member) < 0x1000)    
+
+    UInt64 distances[arMemberCount];
+
+    for (UInt32 index = 0; index < arMemberCount; index++) {
+
+	AppleRAIDMember * member = arMembers[index];
+	if (member) {
+
+	    UInt32 memberState = member->getMemberState();
+	    if (memberState == kAppleRAIDMemberStateOpen || memberState == kAppleRAIDMemberStateClosing) {
+
+//		UInt64 distance = (arLastSeek[index] <= byteStart) ? (byteStart - arLastSeek[index]) : 0xfffffffffffffffeULL;  // elevator
+		UInt64 distance = max(arLastSeek[index], byteStart) - min(arLastSeek[index], byteStart);
+//		if (arSkippedIOCount[index] >= (arMaxReadRequestFactor / 2)) distance = 0;
+		if (arSkippedIOCount[index] >= 12) distance = 0;
+		
+		UInt32 sort = index;
+		while (sort) {
+
+		    if (isOnline(activeMembers[sort-1]) && distance > distances[sort-1]) break;
+
+		    activeMembers[sort] = activeMembers[sort-1];
+		    distances[sort] = distances[sort-1];
+
+		    sort--;
+		}
+		activeMembers[sort] = member;
+		distances[sort] = distance;
+		continue;
+	    } 
+	}
+	activeMembers[index] = (AppleRAIDMember *)index;
+	distances[index] = 0xffffffffffffffffULL;
+    }
+
+    assert((arActiveCount != arMemberCount) ? (isOffline(activeMembers[arActiveCount])) : (isOnline(activeMembers[arMemberCount-1])));
+
+    // adjust last seeked to pointers and skipped counts
+    UInt64 balancedBlockCount = arSetBlockSize * arMaxReadRequestFactor;
+    UInt64 perMemberCount = byteCount / balancedBlockCount / arActiveCount * balancedBlockCount;
+    UInt64 count = 0;
+
+    for (UInt32 virtualIndex = 0; virtualIndex < arActiveCount; virtualIndex++) {
+
+	AppleRAIDMember * member = activeMembers[virtualIndex];
+	if (isOffline(member)) break;
+	UInt32 memberIndex = member->getMemberIndex();
+
+	count = perMemberCount ? min(byteCount, perMemberCount) : min(byteCount, balancedBlockCount);
+	if (count) {
+	    byteStart += count;
+	    byteCount -= count;
+	    arLastSeek[memberIndex] = byteStart;
+	    arSkippedIOCount[memberIndex] = 0;
+	} else {
+	    arSkippedIOCount[memberIndex]++;
+	}
+    }
+    assert(byteCount == 0);
+
+#ifdef DEBUG2
+    static UInt32 sumCount = 0, skippedSum0 = 0, skippedSum1 = 0, overflowCount = 0;
+    static UInt64 averageSeekSum = 0;
+
+    skippedSum0 += arSkippedIOCount[0];
+    skippedSum1 += arSkippedIOCount[1];
+    averageSeekSum += distances[0];
+    if (perMemberCount) overflowCount++;
+    if (sumCount++ >= 99) {
+	printf("skip0=%ld skip1=%ld, over=%ld, lastseek0=%llx lastseek1=%llx, avseek=%llx\n",
+	       skippedSum0, skippedSum1, overflowCount,
+	       arLastSeek[0], arLastSeek[1], averageSeekSum/100);
+	sumCount = skippedSum0 = skippedSum1 = overflowCount = 0;
+	averageSeekSum = 0;
+    }
+#endif    
+}
+
+//8888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888
+//8888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888
+//8888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888888
+
 void AppleRAIDMirrorSet::completeRAIDRequest(AppleRAIDStorageRequest *storageRequest)
 {
     UInt32		cnt;
@@ -218,7 +333,7 @@ void AppleRAIDMirrorSet::completeRAIDRequest(AppleRAIDStorageRequest *storageReq
 	// Ignore missing members.
 	if (arMembers[cnt] == 0) continue;
 
-	// rebuild members
+	// rebuilding members
 	if (arMembers[cnt]->getMemberState() == kAppleRAIDMemberStateRebuilding) {
 
 	    if (!isWrite) {
@@ -710,7 +825,7 @@ void AppleRAIDMirrorSet::rebuild()
     if (sourceOpen) close(this, 0);
     if (targetOpen) target->close(this, 0);
 
-    // if the target state went back to spare that mean the member is being removed from the set
+    // if the target state went back to spare that means the member is being removed from the set
     bool aborting = target->getMemberState() == kAppleRAIDMemberStateSpare;
     if (aborting) target->changeMemberState(kAppleRAIDMemberStateBroken);
 
@@ -816,7 +931,6 @@ bool AppleRAIDMirrorMemoryDescriptor::initWithStorageRequest(AppleRAIDStorageReq
 {
     if (!super::initWithStorageRequest(storageRequest, memberIndex)) return false;
     
-    mdMemberCount = storageRequest->srMemberCount;
     mdSetBlockSize = storageRequest->srSetBlockSize;
     
     return true;
@@ -824,64 +938,88 @@ bool AppleRAIDMirrorMemoryDescriptor::initWithStorageRequest(AppleRAIDStorageReq
 
 bool AppleRAIDMirrorMemoryDescriptor::configureForMemoryDescriptor(IOMemoryDescriptor *memoryDescriptor, UInt64 byteStart, UInt32 activeIndex)
 {
-    UInt32 raidBlockStop, raidBlockEndOffset;
-    UInt32 virtualMemberIndex, startMember, stopMember;
-    UInt32 blockCount, extraBlocks, memberBlockCount, memberBlockStart;
     UInt32 byteCount = memoryDescriptor->getLength();
+    UInt32 blockCount, memberBlockCount;
+    UInt64 setBlockStop, memberBlockStart;
+    UInt32 extraBlocks, setBlockStopOffset;
+    UInt32 startIndex, virtualIndex;
     UInt32 activeCount = mdStorageRequest->srActiveCount;
     
     _direction = memoryDescriptor->getDirection();
     
     if (_direction == kIODirectionOut) {
-        mdMemberByteStart = byteStart;
-        _length = byteCount;
+	mdMemberByteStart = byteStart;
+	_length = byteCount;
     } else {
-        mdSetBlockStart		= byteStart / mdSetBlockSize;
-        mdSetBlockOffset	= byteStart % mdSetBlockSize;
-        startMember		= mdSetBlockStart % activeCount;
-        raidBlockStop		= (byteStart + byteCount - 1) / mdSetBlockSize;
-        raidBlockEndOffset	= (byteStart + byteCount - 1) % mdSetBlockSize;
-        stopMember		= raidBlockStop % activeCount;
-        blockCount		= raidBlockStop - mdSetBlockStart + 1;
-        memberBlockCount	= blockCount / activeCount;
-        extraBlocks		= blockCount % activeCount;
-        virtualMemberIndex	= (activeCount + activeIndex - startMember) % activeCount;
-	memberBlockStart	= mdSetBlockStart + virtualMemberIndex * memberBlockCount + min(virtualMemberIndex, extraBlocks);
+	mdSetBlockStart		= byteStart / mdSetBlockSize;
+	mdSetBlockOffset	= byteStart % mdSetBlockSize;
+	setBlockStop		= (byteStart + byteCount - 1) / mdSetBlockSize;
+	setBlockStopOffset	= (byteStart + byteCount - 1) % mdSetBlockSize;
+	blockCount		= setBlockStop - mdSetBlockStart + 1;
+	memberBlockCount	= blockCount / activeCount;
+	extraBlocks		= blockCount % activeCount;
+	startIndex		= mdSetBlockStart % activeCount;
+
+	// per member stuff
+
+	// find our index relative to this starting member for this request
+	virtualIndex = (activeCount + activeIndex - startIndex) % activeCount;
+	memberBlockStart = mdSetBlockStart + virtualIndex * memberBlockCount + min(virtualIndex, extraBlocks);
+	if (virtualIndex < extraBlocks) memberBlockCount++;
+
+	// find the transfer size for this member
+	mdMemberByteStart = memberBlockStart * mdSetBlockSize;
+	_length = memberBlockCount * mdSetBlockSize;
+
+	// adjust for the starting inter-block offset
+	if (virtualIndex == 0) {
+	    mdMemberByteStart += mdSetBlockOffset;   // XXX same as byteStart?
+	    _length	      -= mdSetBlockOffset;
+	}
+
+	// adjust for ending inter-block offset
+	if (virtualIndex == min(blockCount - 1, activeCount - 1)) _length -= mdSetBlockSize - setBlockStopOffset - 1;
 
-	if (virtualMemberIndex < extraBlocks) memberBlockCount++;
-        
-        mdMemberByteStart = (UInt64)memberBlockStart * mdSetBlockSize;
-        _length = memberBlockCount * mdSetBlockSize;
-        
-        if (virtualMemberIndex == 0) {
-            mdMemberByteStart += mdSetBlockOffset;
-            _length -= mdSetBlockOffset;
-        }
-        
-        if (virtualMemberIndex == min(blockCount - 1, activeCount - 1)) _length -= mdSetBlockSize - raidBlockEndOffset - 1;
-	
 	IOLogRW("mirror activeIndex = %ul, mdMemberByteStart = %llu _length =0x%lx\n", (int)activeIndex, mdMemberByteStart, _length);
     }
     
     mdMemoryDescriptor = memoryDescriptor;
-        
+	
     return _length != 0;
 }
 
 IOPhysicalAddress AppleRAIDMirrorMemoryDescriptor::getPhysicalSegment(IOByteCount offset, IOByteCount *length)
 {
-    UInt32		memberBlockNumber, memberBlockOffset, raidBlockNumber;
-    IOByteCount		raidOffset = offset;
+    IOByteCount		setOffset = offset;
     IOPhysicalAddress	physAddress;
-    
+    UInt32		memberBlockStart, memberBlockOffset, blockCount;
+
     if (_direction != kIODirectionOut) {
-        memberBlockNumber = (mdMemberByteStart + offset) / mdSetBlockSize;
+        memberBlockStart = (mdMemberByteStart + offset) / mdSetBlockSize;
         memberBlockOffset = (mdMemberByteStart + offset) % mdSetBlockSize;
-        raidBlockNumber = memberBlockNumber - mdSetBlockStart;
-        raidOffset = raidBlockNumber * mdSetBlockSize + memberBlockOffset - mdSetBlockOffset;
+        blockCount = memberBlockStart - mdSetBlockStart;
+        setOffset = blockCount * mdSetBlockSize + memberBlockOffset - mdSetBlockOffset;
     }
     
-    physAddress = mdMemoryDescriptor->getPhysicalSegment(raidOffset, length);
+    physAddress = mdMemoryDescriptor->getPhysicalSegment(setOffset, length);
+    
+    return physAddress;
+}
+
+addr64_t AppleRAIDMirrorMemoryDescriptor::getPhysicalSegment64(IOByteCount offset, IOByteCount *length)
+{
+    IOByteCount		setOffset = offset;
+    addr64_t		physAddress;
+    UInt32		memberBlockStart, memberBlockOffset, blockCount;
+
+    if (_direction != kIODirectionOut) {
+        memberBlockStart = (mdMemberByteStart + offset) / mdSetBlockSize;
+        memberBlockOffset = (mdMemberByteStart + offset) % mdSetBlockSize;
+        blockCount = memberBlockStart - mdSetBlockStart;
+        setOffset = blockCount * mdSetBlockSize + memberBlockOffset - mdSetBlockOffset;
+    }
+    
+    physAddress = mdMemoryDescriptor->getPhysicalSegment64(setOffset, length);
     
     return physAddress;
 }
